﻿Imports System.ComponentModel

Public Class CustomerData

    ''' <summary>
    ''' Customer class.
    ''' </summary>
    Class Customer
        Public CustName As String = Nothing
        Public CustOrders As IBindingList = Nothing
    End Class

    ''' <summary>
    ''' Order class.
    ''' </summary>
    Class Order
        Public OrderDate As DateTime
        Public OrderItems As IBindingList
    End Class

    ''' <summary>
    ''' OrderItem class.
    ''' </summary>
    Class OrderItem
        Public ItemQuantity As Integer
        Public ItemDescription As String
    End Class


    ''' <summary>
    ''' CustomerList class: A collection of Customer objects that can be assigned
    ''' as a grid's DataSource.
    ''' </summary>
    Class CustomerList
        Inherits ArrayList
        Implements ITypedList

        Private Function ITypedList_GetItemProperties(listAccessors As PropertyDescriptor()) As PropertyDescriptorCollection Implements ITypedList.GetItemProperties
            If listAccessors Is Nothing Then
                ' Return the property descriptors for top-level rows
                Return New PropertyDescriptorCollection(New PropertyDescriptor() {New MyPropertyDescriptor("CustName"), New MyPropertyDescriptor("CustOrders")})
            Else
                ' Return the property descriptors for second-level and third-level rows
                Dim parentDescriptorName As String = listAccessors(listAccessors.Length - 1).Name
                Select Case parentDescriptorName
                    Case "CustOrders"
                        Return New PropertyDescriptorCollection(New PropertyDescriptor() {New MyPropertyDescriptor("OrderDate"), New MyPropertyDescriptor("OrderItems")})

                    Case "OrderItems"
                        Return New PropertyDescriptorCollection(New PropertyDescriptor() {New MyPropertyDescriptor("ItemQuantity"), New MyPropertyDescriptor("ItemDescription")})
                    Case Else

                        Throw New Exception("Not implemented: " & parentDescriptorName)
                End Select
            End If
        End Function

        Private Function ITypedList_GetListName(listAccessors As PropertyDescriptor()) As String Implements ITypedList.GetListName
            Return ""
        End Function
    End Class

    ''' <summary>
    ''' Property descriptor implementation.
    ''' Store only the name in the base class, then override key methods.
    ''' Overridden methods look up the name in the base class to decide
    ''' how to carry out each property descriptor function.
    ''' </summary>
    Class MyPropertyDescriptor
        Inherits PropertyDescriptor

        Public Sub New(descriptorName As String)
            MyBase.New(descriptorName, New Attribute(0) {})
        End Sub

        Public Overrides ReadOnly Property ComponentType() As Type
            Get
                Select Case Me.Name
                    Case "CustName"
                        Return GetType(Customer)
                    Case "CustOrders"
                        Return GetType(Customer)
                    Case "OrderDate"
                        Return GetType(Order)
                    Case "OrderItems"
                        Return GetType(Order)
                    Case "ItemQuantity"
                        Return GetType(OrderItem)
                    Case "ItemDescription"
                        Return GetType(OrderItem)
                    Case Else
                        Return Nothing
                End Select
            End Get
        End Property

        Public Overrides ReadOnly Property PropertyType() As Type
            Get
                Select Case Me.Name
                    Case "CustName"
                        Return GetType(String)
                    Case "CustOrders"
                        Return GetType(ArrayList)
                    Case "OrderDate"
                        Return GetType(DateTime)
                    Case "OrderItems"
                        Return GetType(ArrayList)
                    Case "ItemQuantity"
                        Return GetType(Integer)
                    Case "ItemDescription"
                        Return GetType(String)
                    Case Else
                        Return Nothing
                End Select
            End Get
        End Property

        Public Overrides Function GetValue(component As Object) As Object
            Select Case Me.Name
                Case "CustName"
                    Return DirectCast(component, Customer).CustName
                Case "CustOrders"
                    Return DirectCast(component, Customer).CustOrders
                Case "OrderDate"
                    Return DirectCast(component, Order).OrderDate
                Case "OrderItems"
                    Return DirectCast(component, Order).OrderItems
                Case "ItemQuantity"
                    Return DirectCast(component, OrderItem).ItemQuantity
                Case "ItemDescription"
                    Return DirectCast(component, OrderItem).ItemDescription
                Case Else
                    Return Nothing
            End Select
        End Function

        Public Overrides ReadOnly Property DisplayName() As String
            Get
                Select Case Me.Name
                    Case "CustName"
                        Return "Customer name"
                    Case "CustOrders"
                        Return "Customer orders"
                    Case "OrderDate"
                        Return "Order date"
                    Case "OrderItems"
                        Return "Order items"
                    Case "ItemQuantity"
                        Return "Quantity"
                    Case "ItemDescription"
                        Return "Description"
                    Case Else
                        Return Nothing
                End Select
            End Get
        End Property

        Public Overrides ReadOnly Property IsReadOnly() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides Sub SetValue(component As Object, value As Object)
            Throw New Exception("Not implemented.")
        End Sub

        Public Overrides Sub ResetValue(component As Object)
            Throw New Exception("Not implemented.")
        End Sub

        Public Overrides Function CanResetValue(component As Object) As Boolean
            Throw New Exception("Not implemented.")
        End Function

        Public Overrides Function ShouldSerializeValue(component As Object) As Boolean
            Throw New Exception("Not implemented.")
        End Function

    End Class

End Class
